import assert from 'assert';
import { describe, it } from 'mocha';
import { TinyQueue } from './TinyQueue';

describe('TinyQueue', function() {
    it('should handle put then take', async function() {
        const tq = new TinyQueue<number>();
        await tq.put(42);
        const elt = await tq.take();
        assert.strictEqual(elt, 42);
    });

    it('should handle take then put', async function() {
        const tq = new TinyQueue<number>();
        const takePromise = tq.take();
        const putPromise = tq.put(42);
        const [elt, _] = await Promise.all([takePromise, putPromise]);
        assert.strictEqual(elt, 42);
    });

    it('should handle N puts then N takes', async function() {
        const tq = new TinyQueue<number>();        
        const N = 3;

        const puts = [];
        for (let i = 0; i < N; ++i) puts.push(tq.put(i));
        
        const takes = [];
        for (let i = 0; i < N; ++i) takes.push(tq.take());
        
        await Promise.all(puts);
        const results = await Promise.all(takes);
        for (let i = 0; i < N; ++i) assert.strictEqual(results[i], i);
    });

    it('should handle N takes then N puts', async function() {
        const tq = new TinyQueue<number>();        
        const N = 3;

        const takes = [];
        for (let i = 0; i < N; ++i) takes.push(tq.take());
        
        const puts = [];
        for (let i = 0; i < N; ++i) puts.push(tq.put(i));
        
        await Promise.all(puts);
        const results = await Promise.all(takes);
        for (let i = 0; i < N; ++i) assert.strictEqual(results[i], i);
    });
});
